package com.arubanetworks.meridiansamples

import android.app.Activity
import android.content.Intent
import android.graphics.Matrix
import android.graphics.PointF
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.appcompat.app.AlertDialog
import com.arubanetworks.meridian.editor.Placemark
import com.arubanetworks.meridian.location.LocationRequest
import com.arubanetworks.meridian.location.LocationRequest.LocationRequestListener
import com.arubanetworks.meridian.location.MeridianLocation
import com.arubanetworks.meridian.location.MeridianOrientation
import com.arubanetworks.meridian.maps.ClusteredMarker
import com.arubanetworks.meridian.maps.MapView
import com.arubanetworks.meridian.maps.Marker
import com.arubanetworks.meridian.maps.directions.*
import com.arubanetworks.meridian.maps.directions.Directions.DirectionsRequestListener
import com.arubanetworks.meridian.search.SearchActivity

/**
 * Demonstrates the use of the MapView.  It is recommended to use the SDK map fragment instead of the mapview
 */

class MapViewFragment : androidx.fragment.app.Fragment(), MapView.DirectionsEventListener, MapView.MapEventListener, MapView.MarkerEventListener {
    private var mapView: MapView? = null
    private var directions: Directions? = null
    private var locationRequest: LocationRequest? = null

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        val layout = inflater.inflate(R.layout.fragment_mapview, container, false)
        mapView = layout.findViewById(R.id.demo_mapview)
        mapView?.appKey = Application.APP_KEY

        // If you want to handle MapView events
        mapView?.setMapEventListener(this)

        // If you want to handle directions events
        mapView?.setDirectionsEventListener(this)

        // Sample of how to set the direction step colors
        /*
        val ctx: Context? = context?.applicationContext
        if (ctx != null){
            // Sets the Default colors
            var activeColor: Int? = ContextCompat.getColor(ctx, R.color.direction_active)
            var inactiveColor: Int? = ContextCompat.getColor(ctx, R.color.direction_inactive)
            if (activeColor != null && inactiveColor != null) {
                mapView?.setDirectionPathOptions(activeColor, inactiveColor, 1.0f)
            }
            // Sets a RED and GREEN path
            var redSolid: Int? = ContextCompat.getColor(ctx, R.color.direction_red_solid)
            var greenTransparent: Int? = ContextCompat.getColor(ctx, R.color.direction_green_transparent)
            if (redSolid != null && greenTransparent != null) {
                mapView?.setDirectionPathOptions(redSolid, greenTransparent, 1.0f)
            }
        }
         */

        // If you want to handle marker events
        mapView?.setMarkerEventListener(this)

        // Set map options if desired
        val mapOptions = mapView?.options
        mapOptions?.HIDE_MAP_LABEL = true
        mapView?.options = mapOptions

        // Set which map to load
        // It is recommended to do this after setting the map options
        mapView?.mapKey = Application.MAP_KEY

        // Demonstration of how to customize the mapView's locationMarker:
        //    change default color for Bluetooth to orange
        //    modify the name
        //    modify the details
        //    alternatively... hide the call-out entirely
        //
        // val lm = mapView?.getLocationMarker()
        // lm?.setCustomColor(LocationMarker.State.BLUETOOTH, 0xFFFF7700.toInt())
        // lm?.name = "Current Location Label"
        // lm?.details = "Details"
        // // lm?.setShowsCallout(false)

        return layout
    }

    override fun onResume() {
        super.onResume()
        mapView?.onResume()
    }

    override fun onPause() {
        super.onPause()
        mapView?.onPause()
    }

    override fun onDestroy() {
        super.onDestroy()
        mapView?.onDestroy()
    }


    //
    // MapViewListener methods
    //
    override fun onMapLoadStart() {}
    override fun onMapLoadFinish() {}
    override fun onPlacemarksLoadFinish() {
        // example: highlight the first four placemarks
        /*
        val markerList = ArrayList<Marker>()
        var index = 0
        if (mapView?.getAllMarkers() != null) {
            for (marker in mapView!!.getAllMarkers()) {
                markerList.add(marker)
                index++
                if (index >= 4) {
                    val highlightedMarkers = HighlightedMarkers.Builder(markerList).build()
                    mapView!!.commitTransaction(Transaction.Builder().setAnimationDuration(500).addMarker(highlightedMarkers).build())
                    break
                }
            }
        }
        */
    }
    override fun onMapRenderFinish() {}
    override fun onMapLoadFail(tr: Throwable) {}
    override fun onMapTransformChange(transform: Matrix) {}
    override fun onLocationUpdated(meridianLocation: MeridianLocation) {}
    override fun onOrientationUpdated(orientation: MeridianOrientation) {}

    //
    // DirectionsEventListener methods
    //
    override fun onDirectionsReroute() {}
    override fun onDirectionsClick(marker: Marker): Boolean {
        if (activity != null) {
            val p = mapView!!.getAssociatedPlacemark(marker)
            if (p != null) {
                startDirections(DirectionsDestination.forPlacemarkKey(p.key))
            } else {
                AlertDialog.Builder(requireActivity())
                        .setMessage("Directions only implemented for placemarks.")
                        .setPositiveButton("OK", null)
                        .show()
            }
        }
        return false
    }
    override fun onDirectionsStart(): Boolean {return false}
    override fun onRouteStepIndexChange(index: Int): Boolean {return false}
    override fun onDirectionsClosed(): Boolean {return false}
    override fun onDirectionsError(tr: Throwable): Boolean {return false}
    override fun onUseAccessiblePathsChange() {}
    //
    // MarkerEventListener methods
    //
    override fun onMarkerSelect(marker: Marker): Boolean {
        // prevent clustered markers from being selected
        return marker is ClusteredMarker
    }
    override fun onMarkerDeselect(marker: Marker): Boolean {return false}
    override fun markerForPlacemark(placemark: Placemark): Marker? {return null}
    override fun markerForSelectedMarker(markerToSelect: Marker): Marker? {return null}
    override fun onCalloutClick(marker: Marker): Boolean {return false}

    // Directions support
    /**
     * Start directions to the given placemark.
     */
    private fun startDirections(destination: DirectionsDestination) {
        // check if we already have started directions
        directions?.cancel()

        // if we have requested the user location and its still running, keep it
        if (locationRequest != null && locationRequest!!.isRunning()) return
        if (activity == null) {
            return
        }
        mapView!!.onDirectionsRequestStart()

        // Lets see if we can get the users location
        locationRequest = LocationRequest.requestCurrentLocation(activity, Application.APP_KEY, object : LocationRequestListener {
            override fun onResult(location: MeridianLocation?) {
                if (location == null) {
                    startSearchActivity(destination)
                    return
                }
                // Looks like we got a good location
                onSourceResult(destination, DirectionsSource.forMapPoint(location.mapKey, location.point))
            }

            override fun onError(errorType: LocationRequest.ErrorType) {
                if (errorType != LocationRequest.ErrorType.CANCELED) {
                    startSearchActivity(destination)
                }
            }
        })
    }

    private fun startSearchActivity(destination: DirectionsDestination?) {
        // Location is unknown so use the Search activity to get a
        // start location from the user.

        // Handle any exclusions
        val i = SearchActivity.createIntent(activity, Application.APP_KEY,
                destination?.searchExclusions)
        i.putExtra(PENDING_DESTINATION_KEY, destination)
        startActivityForResult(i, SOURCE_REQUEST_CODE)
    }

    private fun startDirections(destination: DirectionsDestination, source: DirectionsSource) {
        directions?.cancel()
        if (activity == null) {
            return
        }
        directions = Directions.Builder()
                .setAppKey(Application.APP_KEY)
                .setDestination(destination)
                .setListener(object : DirectionsRequestListener {
                    override fun onDirectionsRequestStart() {
                        if (mapView != null) {
                            mapView!!.onDirectionsRequestStart()
                        }
                    }

                    override fun onDirectionsRequestComplete(response: DirectionsResponse) {
                        if (mapView != null) {
                            mapView!!.onDirectionsRequestComplete(response)
                        }
                    }

                    override fun onDirectionsRequestError(th: Throwable) {
                        if (mapView != null) {
                            mapView!!.onDirectionsRequestError(th)
                        }
                    }

                    override fun onDirectionsRequestCanceled() {
                        if (mapView != null) {
                            mapView!!.onDirectionsRequestCanceled()
                        }
                    }
                })
                .setTransportType(TransportType.WALKING)
                .setSource(source).build()
        directions?.calculate()
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        if (requestCode == SOURCE_REQUEST_CODE) {
            if (resultCode == Activity.RESULT_OK) {
                val destination = data?.getSerializableExtra(PENDING_DESTINATION_KEY) as DirectionsDestination
                val result = SearchActivity.getSearchResult(data).placemark
                val source : DirectionsSource
                if (result.isInvalid) {
                    source = DirectionsSource.forPlacemarkKey(result.key)
                } else {
                    source = DirectionsSource.forMapPoint(result.key.parent, PointF(result.x, result.y))
                }
                onSourceResult(destination, source)
            }
            if (mapView != null) {
                mapView!!.onDirectionsRequestCanceled()
            }
        }
        super.onActivityResult(requestCode, resultCode, data)
    }

    private fun onSourceResult(destination: DirectionsDestination, source: DirectionsSource) {
        startDirections(destination, source)
    }
    companion object {
        private const val PENDING_DESTINATION_KEY = "meridianSamples.PendingDestinationKey"
        private val SOURCE_REQUEST_CODE = "meridianSamples.source_request".hashCode() and 0xFF
    }
}
