package com.arubanetworks.meridiansamples

import android.os.Bundle
import androidx.fragment.app.Fragment
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.*
import android.widget.Button
import android.widget.FrameLayout

import com.arubanetworks.meridian.editor.EditorKey
import com.arubanetworks.meridian.triggers.ProximityBeacon
import com.arubanetworks.meridian.triggers.TriggersService
import com.arubanetworks.meridiansamples.TriggersReceiver.TriggerEvent
import android.widget.Toast

import android.widget.TextView
import com.arubanetworks.meridiansamples.Application.Companion.PROXIMITY_BEACON_MAC

class TriggersFragment : Fragment() {

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {

        val arg: Bundle? = arguments
        val appKey: EditorKey? = arg?.getSerializable(APP_KEY) as EditorKey
        if (appKey == null) {
            Toast.makeText(activity, "No EditorKey passed to Fragment!", Toast.LENGTH_LONG).show()
            return FrameLayout(requireActivity())
        }

        val view = inflater.inflate(R.layout.fragment_triggers, container,false)
        val startButton = view.findViewById<Button>(R.id.start_button)
        startButton?.setOnClickListener {
            val subscription = if (PROXIMITY_BEACON_MAC.isNotEmpty()) {
                val beacon = ProximityBeacon(PROXIMITY_BEACON_MAC)
                TriggersService.TriggerSubscription("KotlinTest", arrayListOf(beacon),45)
            } else {
                TriggersService.TriggerSubscription("KotlinTestAllBeacons",45)
            }
            TriggersService.startMonitoring(requireActivity(), appKey, arrayListOf(subscription))
            addLogMessage("Start")
        }
        val stopButton = view.findViewById<Button>(R.id.stop_button)
        stopButton?.setOnClickListener {
            TriggersService.stopMonitoring(requireActivity())
            addLogMessage("Stop")
        }
        val resetButton = view.findViewById<Button>(R.id.reset_button)
        resetButton?.setOnClickListener {
            TriggersService.resetAllTriggers(requireActivity())
            addLogMessage("Reset")
        }
        return view
    }

    override fun onResume() {
        super.onResume()
        TriggerEvent.getInstance().register(this)
    }

    override fun onPause() {
        super.onPause()
        TriggerEvent.getInstance().unregister(this)
    }

    private fun addLogMessage(message: String) {
        val v = view ?: return
        val tv = v.findViewById<TextView>(R.id.triggers_log)
        val sv = v.findViewById<ScrollView>(R.id.triggers_scroll_log)
        if (tv == null || sv == null) return
        tv.append("\n" + message)
        if (tv.bottom - (sv.height + sv.scrollY) <= 0)
            sv.post { sv.fullScroll(ScrollView.FOCUS_DOWN) }
    }

    @com.squareup.otto.Subscribe
    fun onTriggerEvent(event: TriggerEvent) {
        addLogMessage(message = "Received " + event.name + " from " + event.major + "/" + event.minor)
    }

    companion object {

        private const val APP_KEY = "TriggersFragment.AppKey"

        fun newInstance(appKey: EditorKey): TriggersFragment {
            val fragment = TriggersFragment()
            if (appKey.parent != null) throw IllegalArgumentException("appKey must have null parent.")
            var b = fragment.arguments
            if (b == null) b = Bundle()
            b.putSerializable(APP_KEY, appKey)
            fragment.arguments = b
            return fragment
        }
    }
}
